'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var reactDom = require('@floating-ui/react-dom');
var reactDom$1 = require('react-dom');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

var index = typeof document !== 'undefined' ? React.useLayoutEffect : React.useEffect;

function createPubSub() {
  const map = new Map();
  return {
    emit(event, data) {
      var _map$get;

      (_map$get = map.get(event)) == null ? void 0 : _map$get.forEach(handler => handler(data));
    },

    on(event, listener) {
      map.set(event, [...(map.get(event) || []), listener]);
    },

    off(event, listener) {
      map.set(event, (map.get(event) || []).filter(l => l !== listener));
    }

  };
}

let globalId = 0;

function genId() {
  return "floating-ui-" + globalId++;
}

function useFloatingId() {
  return React.useMemo(genId, []);
}

const FloatingNodeContext = /*#__PURE__*/React.createContext(null);
const FloatingTreeContext = /*#__PURE__*/React.createContext(null);
const useFloatingParentNodeId = () => {
  var _useContext$id, _useContext;

  return (_useContext$id = (_useContext = React.useContext(FloatingNodeContext)) == null ? void 0 : _useContext.id) != null ? _useContext$id : null;
};
const useFloatingTree = () => React.useContext(FloatingTreeContext);
/**
 * Registers a node into the floating tree, returning its id.
 */

const useFloatingNodeId = () => {
  const id = useFloatingId();
  const tree = useFloatingTree();
  const parentId = useFloatingParentNodeId();
  index(() => {
    const node = {
      id,
      parentId
    };
    tree == null ? void 0 : tree.addNode(node);
    return () => {
      tree == null ? void 0 : tree.removeNode(node);
    };
  }, [tree, id, parentId]);
  return id;
};
/**
 * Provides parent node context for nested floating elements.
 * @see https://floating-ui.com/docs/FloatingTree
 */

const FloatingNode = _ref => {
  let {
    children,
    id
  } = _ref;
  const parentId = useFloatingParentNodeId();
  return /*#__PURE__*/React__default["default"].createElement(FloatingNodeContext.Provider, {
    value: {
      id,
      parentId
    }
  }, children);
};
/**
 * Provides context for nested floating elements when they are not children of
 * each other on the DOM (i.e. portalled to a common node, rather than their
 * respective parent).
 * @see https://floating-ui.com/docs/FloatingTree
 */

const FloatingTree = _ref2 => {
  let {
    children
  } = _ref2;
  const nodesRef = React.useRef([]);
  const addNode = React.useCallback(node => {
    nodesRef.current = [...nodesRef.current, node];
  }, []);
  const removeNode = React.useCallback(node => {
    nodesRef.current = nodesRef.current.filter(n => n !== node);
  }, []);
  return /*#__PURE__*/React__default["default"].createElement(FloatingTreeContext.Provider, {
    value: {
      nodesRef,
      addNode,
      removeNode,
      events: createPubSub()
    }
  }, children);
};

function reducePropsToArrayWithInitValue(userProps) {
  return Object.entries(userProps != null ? userProps : {}).reduce((acc, _ref) => {
    let [key, value] = _ref;
    acc[key] = [value];
    return acc;
  }, {});
}

function mergeProps(userProps, propsList, elementKey) {
  const mergePropsMap = {
    reference: elementKey === 'reference' ? reducePropsToArrayWithInitValue(userProps) : {},
    floating: elementKey === 'floating' ? reducePropsToArrayWithInitValue(userProps) : {}
  };
  propsList.forEach(props => {
    var _props$elementKey;

    const elementProps = (_props$elementKey = props == null ? void 0 : props[elementKey]) != null ? _props$elementKey : {};
    Object.keys(elementProps).forEach(propKey => {
      if (typeof elementProps[propKey] === 'function') {
        var _mergePropsMap$elemen;

        if (mergePropsMap[elementKey][propKey] == null) {
          mergePropsMap[elementKey][propKey] = [];
        }

        (_mergePropsMap$elemen = mergePropsMap[elementKey][propKey]) == null ? void 0 : _mergePropsMap$elemen.push(elementProps[propKey]);
      }
    });
  });
  return { ...(elementKey === 'floating' && {
      tabIndex: -1
    }),
    ...userProps,
    ...propsList.reduce((acc, props) => {
      Object.assign(acc, props == null ? void 0 : props[elementKey]);
      return acc;
    }, {}),
    ...Object.entries(mergePropsMap[elementKey]).reduce((acc, _ref2) => {
      let [prop, fns] = _ref2;

      if (prop.indexOf('on') === 0) {
        acc[prop] = function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          fns.forEach(fn => fn(...args));
        };
      }

      return acc;
    }, {})
  };
}

const useInteractions = function (propsList) {
  if (propsList === void 0) {
    propsList = [];
  }

  return {
    getReferenceProps: userProps => mergeProps(userProps, propsList, 'reference'),
    getFloatingProps: userProps => mergeProps(userProps, propsList, 'floating')
  };
};

var pointInPolygon$1 = {exports: {}};

var flat = function pointInPolygonFlat (point, vs, start, end) {
    var x = point[0], y = point[1];
    var inside = false;
    if (start === undefined) start = 0;
    if (end === undefined) end = vs.length;
    var len = (end-start)/2;
    for (var i = 0, j = len - 1; i < len; j = i++) {
        var xi = vs[start+i*2+0], yi = vs[start+i*2+1];
        var xj = vs[start+j*2+0], yj = vs[start+j*2+1];
        var intersect = ((yi > y) !== (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    return inside;
};

// ray-casting algorithm based on
// https://wrf.ecse.rpi.edu/Research/Short_Notes/pnpoly.html

var nested = function pointInPolygonNested (point, vs, start, end) {
    var x = point[0], y = point[1];
    var inside = false;
    if (start === undefined) start = 0;
    if (end === undefined) end = vs.length;
    var len = end - start;
    for (var i = 0, j = len - 1; i < len; j = i++) {
        var xi = vs[i+start][0], yi = vs[i+start][1];
        var xj = vs[j+start][0], yj = vs[j+start][1];
        var intersect = ((yi > y) !== (yj > y))
            && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
        if (intersect) inside = !inside;
    }
    return inside;
};

var pointInPolygonFlat = flat;
var pointInPolygonNested = nested;

pointInPolygon$1.exports = function pointInPolygon (point, vs, start, end) {
    if (vs.length > 0 && Array.isArray(vs[0])) {
        return pointInPolygonNested(point, vs, start, end);
    } else {
        return pointInPolygonFlat(point, vs, start, end);
    }
};
pointInPolygon$1.exports.nested = pointInPolygonNested;
pointInPolygon$1.exports.flat = pointInPolygonFlat;

var pointInPolygon = pointInPolygon$1.exports;

function getDocument(floating) {
  var _floating$ownerDocume;

  return (_floating$ownerDocume = floating == null ? void 0 : floating.ownerDocument) != null ? _floating$ownerDocume : document;
}

function getWindow(value) {
  var _getDocument$defaultV;

  return (_getDocument$defaultV = getDocument(value).defaultView) != null ? _getDocument$defaultV : window;
}

function isElement(value) {
  return value ? value instanceof getWindow(value).Element : false;
}
function isHTMLElement(value) {
  return value ? value instanceof getWindow(value).HTMLElement : false;
}

function getChildren(tree, id) {
  var _tree$nodesRef$curren;

  let allChildren = (_tree$nodesRef$curren = tree == null ? void 0 : tree.nodesRef.current.filter(node => {
    var _node$context;

    return node.parentId === id && ((_node$context = node.context) == null ? void 0 : _node$context.open);
  })) != null ? _tree$nodesRef$curren : [];
  let currentChildren = allChildren;

  while (currentChildren.length) {
    var _tree$nodesRef$curren2;

    currentChildren = (_tree$nodesRef$curren2 = tree == null ? void 0 : tree.nodesRef.current.filter(node => {
      var _currentChildren;

      return (_currentChildren = currentChildren) == null ? void 0 : _currentChildren.some(n => {
        var _node$context2;

        return node.parentId === n.id && ((_node$context2 = node.context) == null ? void 0 : _node$context2.open);
      });
    })) != null ? _tree$nodesRef$curren2 : [];
    allChildren = allChildren.concat(currentChildren);
  }

  return allChildren;
}

function safePolygon(_temp) {
  let {
    timeout = 0,
    debug = null
  } = _temp === void 0 ? {} : _temp;
  let timeoutId;
  return _ref => {
    let {
      x,
      y,
      placement,
      refs,
      onClose,
      nodeId,
      tree
    } = _ref;
    return function onPointerMove(event) {
      var _refs$floating$curren;

      if (event.pointerType === 'touch') {
        return;
      }

      const {
        target,
        clientX,
        clientY
      } = event;
      const targetNode = target;

      if (event.type === 'pointermove' && isElement(refs.reference.current) && refs.reference.current.contains(targetNode) || (_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(targetNode)) {
        return;
      } // If any child has a menu open, abort


      if (tree && getChildren(tree, nodeId).some(_ref2 => {
        let {
          context
        } = _ref2;
        return context == null ? void 0 : context.open;
      })) {
        return;
      }

      if (!refs.reference.current || !refs.floating.current || placement == null || x == null || y == null) {
        return;
      }

      const refRect = refs.reference.current.getBoundingClientRect();
      const rect = refs.floating.current.getBoundingClientRect();
      const side = placement.split('-')[0];
      const cursorLeaveFromRight = x > rect.right - rect.width / 2;
      const cursorLeaveFromBottom = y > rect.bottom - rect.height / 2;
      const BUFFER = 1; // Within the rectangular trough between the two elements

      switch (side) {
        case 'top':
          if (clientX >= rect.left && clientX <= rect.right && clientY >= rect.bottom && clientY <= refRect.top) {
            return;
          }

          break;

        case 'bottom':
          if (clientX >= rect.left && clientX <= rect.right && clientY >= refRect.bottom && clientY <= rect.top) {
            return;
          }

          break;

        case 'left':
          if (clientX >= rect.right && clientX <= refRect.left && clientY >= rect.left && clientY <= rect.right) {
            return;
          }

          break;

        case 'right':
          if (clientX >= rect.right && clientX <= refRect.left && clientY >= rect.right && clientY <= refRect.left) {
            return;
          }

          break;
      }

      function getPolygon(_ref3) {
        let [x, y] = _ref3;
        const isFloatingWider = rect.width > refRect.width;
        const isFloatingTaller = rect.height > refRect.height;

        switch (side) {
          case 'top':
            {
              const cursorPointOne = [isFloatingWider ? x : cursorLeaveFromRight ? x + BUFFER : x - BUFFER, y + BUFFER];
              const cursorPointTwo = [isFloatingWider ? x : cursorLeaveFromRight ? x - BUFFER : x + BUFFER, y + BUFFER];
              const commonPoints = [[rect.left, cursorLeaveFromRight ? rect.bottom - BUFFER : isFloatingWider ? rect.bottom - BUFFER : rect.top], [rect.right, cursorLeaveFromRight ? isFloatingWider ? rect.bottom - BUFFER : rect.top : rect.bottom - BUFFER]];

              if (cursorLeaveFromRight) {
                return [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return [cursorPointOne, ...commonPoints, cursorPointTwo];
            }

          case 'bottom':
            {
              const cursorPointOne = [isFloatingWider ? x : cursorLeaveFromRight ? x + BUFFER : x - BUFFER, y - BUFFER];
              const cursorPointTwo = [isFloatingWider ? x : cursorLeaveFromRight ? x - BUFFER : x + BUFFER, y - BUFFER];
              const commonPoints = [[rect.left, cursorLeaveFromRight ? rect.top + BUFFER : isFloatingWider ? rect.top + BUFFER : rect.bottom], [rect.right, cursorLeaveFromRight ? isFloatingWider ? rect.top + BUFFER : rect.bottom : rect.top + BUFFER]];

              if (cursorLeaveFromRight) {
                return [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return [cursorPointOne, ...commonPoints, cursorPointTwo];
            }

          case 'left':
            {
              const cursorPointOne = [x + BUFFER, isFloatingTaller ? y : cursorLeaveFromBottom ? y - BUFFER : y + BUFFER];
              const cursorPointTwo = [x + BUFFER, isFloatingTaller ? y : cursorLeaveFromBottom ? y + BUFFER : y - BUFFER];
              const commonPoints = [[cursorLeaveFromBottom ? rect.right - BUFFER : isFloatingTaller ? rect.right - BUFFER : rect.left, rect.top], [cursorLeaveFromBottom ? isFloatingTaller ? rect.right - BUFFER : rect.left : rect.right - BUFFER, rect.bottom]];

              if (cursorLeaveFromBottom) {
                return [cursorPointOne, ...commonPoints, cursorPointTwo];
              }

              return [...commonPoints, cursorPointOne, cursorPointTwo];
            }

          case 'right':
            {
              const cursorPointOne = [x - BUFFER, isFloatingTaller ? y : cursorLeaveFromBottom ? y + BUFFER : y - BUFFER];
              const cursorPointTwo = [x - BUFFER, isFloatingTaller ? y : cursorLeaveFromBottom ? y - BUFFER : y + BUFFER];
              const commonPoints = [[cursorLeaveFromBottom ? rect.left + BUFFER : isFloatingTaller ? rect.left + BUFFER : rect.right, rect.top], [cursorLeaveFromBottom ? isFloatingTaller ? rect.left + BUFFER : rect.right : rect.left + BUFFER, rect.bottom]];

              if (cursorLeaveFromBottom) {
                return [cursorPointOne, cursorPointTwo, ...commonPoints];
              }

              return [cursorPointOne, ...commonPoints, cursorPointTwo];
            }
        }
      }

      const poly = getPolygon([x, y]);

      if (process.env.NODE_ENV !== 'production') {
        debug == null ? void 0 : debug(getPolygon([x, y]).slice(0, 4).join(', '));
      }

      if (!pointInPolygon([clientX, clientY], poly)) {
        clearTimeout(timeoutId);
        onClose();
      } else if (timeout) {
        timeoutId = setTimeout(onClose, timeout);
      }
    };
  };
}

const DEFAULT_ID = 'floating-ui-root';
const FloatingPortalContext = /*#__PURE__*/React.createContext(DEFAULT_ID);
const useFloatingPortalId = () => React.useContext(FloatingPortalContext);
/**
 * Portals your floating element outside of the main app node.
 * @see https://floating-ui.com/docs/FloatingPortal
 */

const FloatingPortal = _ref => {
  let {
    children,
    id = DEFAULT_ID
  } = _ref;
  const [mounted, setMounted] = React.useState(false);
  const portalRef = React.useRef(null);
  index(() => {
    const root = document.getElementById(id);

    if (root) {
      portalRef.current = root;
    } else {
      portalRef.current = document.createElement('div');
      portalRef.current.id = id;
    }

    const el = portalRef.current;

    if (!document.body.contains(el)) {
      document.body.appendChild(el);
    }

    setMounted(true);
  }, []);

  if (mounted && portalRef.current) {
    return /*#__PURE__*/React__default["default"].createElement(FloatingPortalContext.Provider, {
      value: id
    }, /*#__PURE__*/reactDom$1.createPortal(children, portalRef.current));
  }

  return null;
};

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

/**
 * Provides base styling for a fixed overlay element to dim content or block
 * pointer events behind a floating element.
 * It's a regular `<div>`, so it can be styled via any CSS solution you prefer.
 * @see https://floating-ui.com/docs/FloatingOverlay
 */

const FloatingOverlay = /*#__PURE__*/React.forwardRef(function FloatingOverlay(props, ref) {
  return /*#__PURE__*/React__default["default"].createElement("div", _extends({
    ref: ref
  }, props, {
    style: {
      position: 'fixed',
      top: 0,
      right: 0,
      bottom: 0,
      left: 0,
      width: '100%',
      height: '100%',
      ...props.style
    }
  }));
});

function getDelay(value, prop) {
  if (typeof value === 'number') {
    return value;
  }

  return value == null ? void 0 : value[prop];
}

/**
 * Adds hover event listeners that change the open state, like CSS :hover.
 * @see https://floating-ui.com/docs/useHover
 */
const useHover = function (context, _temp) {
  let {
    enabled = true,
    delay = 0,
    handleLeave = null,
    mouseOnly = false,
    pointerRestMs = 0
  } = _temp === void 0 ? {} : _temp;
  const {
    open,
    onOpenChange,
    dataRef,
    events,
    refs
  } = context;
  const tree = useFloatingTree();
  const timeoutRef = React.useRef();
  const handlerRef = React.useRef();
  const pointerRestTimeoutRef = React.useRef();
  const blockPointerMoveRef = React.useRef(true);
  React.useEffect(() => {
    function onDismiss() {
      clearTimeout(timeoutRef.current);
      clearTimeout(pointerRestTimeoutRef.current);
      blockPointerMoveRef.current = true;
    }

    events.on('dismiss', onDismiss);
    return () => {
      events.off('dismiss', onDismiss);
    };
  }, [events]);
  React.useEffect(() => {
    if (!enabled || !handleLeave) {
      return;
    }

    function onLeave() {
      var _dataRef$current$open;

      if ((_dataRef$current$open = dataRef.current.openEvent) != null && _dataRef$current$open.type.includes('pointer')) {
        onOpenChange(false);
      }
    }

    const html = getDocument(refs.floating.current).documentElement;
    html.addEventListener('mouseleave', onLeave);
    return () => {
      html.removeEventListener('mouseleave', onLeave);
    };
  }, [refs.floating, onOpenChange, enabled, handleLeave, dataRef]);

  function closeWithDelay(runElseBranch) {
    if (runElseBranch === void 0) {
      runElseBranch = true;
    }

    if (delay) {
      clearTimeout(timeoutRef.current);
      timeoutRef.current = setTimeout(() => onOpenChange(false), getDelay(delay, 'close'));
    } else if (runElseBranch) {
      onOpenChange(false);
    }
  }

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onPointerMove() {
        if (open || pointerRestMs === 0) {
          return;
        }

        clearTimeout(pointerRestTimeoutRef.current);
        pointerRestTimeoutRef.current = setTimeout(() => {
          if (!blockPointerMoveRef.current) {
            onOpenChange(true);
          }
        }, pointerRestMs);
      },

      onPointerEnter(event) {
        if (open || mouseOnly && event.pointerType !== 'mouse') {
          return;
        }

        blockPointerMoveRef.current = false;
        dataRef.current.openEvent = event.nativeEvent;

        if (delay) {
          clearTimeout(timeoutRef.current);
          timeoutRef.current = setTimeout(() => {
            onOpenChange(true);
          }, getDelay(delay, 'open'));
        } else {
          onOpenChange(true);
        }
      },

      onPointerLeave(event) {
        var _dataRef$current$open2;

        if (((_dataRef$current$open2 = dataRef.current.openEvent) == null ? void 0 : _dataRef$current$open2.type) === 'click') {
          return;
        }

        const doc = getDocument(refs.floating.current);
        clearTimeout(pointerRestTimeoutRef.current);

        if (handleLeave) {
          clearTimeout(timeoutRef.current);
          handlerRef.current && doc.removeEventListener('pointermove', handlerRef.current);
          handlerRef.current = handleLeave({ ...context,
            tree,
            x: event.clientX,
            y: event.clientY,

            onClose() {
              handlerRef.current && doc.removeEventListener('pointermove', handlerRef.current);
              closeWithDelay();
            }

          });
          doc.addEventListener('pointermove', handlerRef.current);
          return;
        }

        closeWithDelay();
      }

    },
    floating: {
      onPointerEnter() {
        clearTimeout(timeoutRef.current);
      },

      onPointerLeave: () => closeWithDelay(false)
    }
  };
};

const FloatingDelayGroupContext = /*#__PURE__*/React.createContext({
  delay: 1000,
  initialDelay: 1000,
  currentId: null,
  setCurrentId: () => {},
  setState: () => {}
});
const useDelayGroupContext = () => React.useContext(FloatingDelayGroupContext);
/**
 * Provides context for a group of floating elements that should share a
 * `delay`.
 * @see https://floating-ui.com/docs/FloatingDelayGroup
 */

const FloatingDelayGroup = _ref => {
  let {
    children,
    delay
  } = _ref;
  const [state, setState] = React.useState({
    delay,
    initialDelay: delay,
    currentId: null
  });
  const setCurrentId = React.useCallback(currentId => {
    setState(state => ({ ...state,
      currentId
    }));
  }, []);
  return /*#__PURE__*/React__default["default"].createElement(FloatingDelayGroupContext.Provider, {
    value: { ...state,
      setState,
      setCurrentId
    }
  }, children);
};
const useDelayGroup = (_ref2, _ref3) => {
  let {
    open,
    onOpenChange
  } = _ref2;
  let {
    id
  } = _ref3;
  const {
    currentId,
    initialDelay,
    setState
  } = useDelayGroupContext();
  React.useEffect(() => {
    if (currentId && onOpenChange) {
      setState(state => ({ ...state,
        delay: {
          open: 0,
          close: getDelay(initialDelay, 'close')
        }
      }));

      if (currentId !== id) {
        onOpenChange(false);
      }
    }
  }, [id, onOpenChange, setState, currentId, initialDelay]);
  React.useEffect(() => {
    if (!open && currentId === id && onOpenChange) {
      onOpenChange(false);
      setState(state => ({ ...state,
        delay: initialDelay,
        currentId: null
      }));
    }
  }, [open, setState, currentId, id, onOpenChange, initialDelay]);
};

/**
 * Adds relevant screen reader props for a given element `role`.
 */
const useRole = function (_ref, _temp) {
  let {
    open
  } = _ref;
  let {
    enabled = true,
    role = 'dialog'
  } = _temp === void 0 ? {} : _temp;
  const rootId = useFloatingId();
  const floatingProps = {
    id: rootId,
    role
  };

  if (!enabled) {
    return {};
  }

  if (role === 'tooltip') {
    return {
      reference: {
        'aria-describedby': open ? rootId : undefined
      },
      floating: floatingProps
    };
  }

  return {
    reference: {
      'aria-expanded': open ? 'true' : 'false',
      'aria-haspopup': role,
      'aria-controls': open ? rootId : undefined,
      ...(role === 'listbox' && {
        role: 'combobox'
      })
    },
    floating: floatingProps
  };
};

/**
 * Adds click event listeners that change the open state (toggle behavior).
 * @see https://floating-ui.com/docs/useClick
 */
const useClick = function (_ref, _temp) {
  let {
    open,
    onOpenChange,
    dataRef
  } = _ref;
  let {
    enabled = true
  } = _temp === void 0 ? {} : _temp;

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onClick(event) {
        if (open) {
          var _dataRef$current$open;

          if (((_dataRef$current$open = dataRef.current.openEvent) == null ? void 0 : _dataRef$current$open.type) === 'click') {
            onOpenChange(false);
          }
        } else {
          onOpenChange(true);
        }

        dataRef.current.openEvent = event.nativeEvent;
      }

    }
  };
};

/**
 * Adds listeners that dismiss (close) the floating element.
 * @see https://floating-ui.com/docs/useDismiss
 */
const useDismiss = function (_ref, _temp) {
  let {
    open,
    onOpenChange,
    refs,
    events,
    nodeId
  } = _ref;
  let {
    enabled = true,
    escapeKey = true,
    outsidePointerDown = true,
    referencePointerDown = false,
    ancestorScroll = false
  } = _temp === void 0 ? {} : _temp;
  const tree = useFloatingTree();
  const focusReference = React.useCallback(() => {
    if (isHTMLElement(refs.reference.current)) {
      refs.reference.current.focus();
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [refs.reference.current]);
  React.useEffect(() => {
    if (!open || !enabled) {
      return;
    }

    function onKeyDown(event) {
      if (event.key === 'Escape') {
        events.emit('dismiss');
        onOpenChange(false);
        focusReference();
      }
    }

    function onPointerDown(event) {
      var _refs$floating$curren;

      const targetIsInsideChildren = tree && getChildren(tree, nodeId).some(node => {
        var _node$context, _node$context$refs$fl;

        return (_node$context = node.context) == null ? void 0 : (_node$context$refs$fl = _node$context.refs.floating.current) == null ? void 0 : _node$context$refs$fl.contains(event.target);
      });

      if ((_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(event.target) || isElement(refs.reference.current) && refs.reference.current.contains(event.target) || targetIsInsideChildren) {
        return;
      }

      events.emit('dismiss');
      onOpenChange(false);
      focusReference();
    }

    function onScroll() {
      onOpenChange(false);
    }

    const doc = getDocument(refs.floating.current);
    escapeKey && doc.addEventListener('keydown', onKeyDown);
    outsidePointerDown && doc.addEventListener('pointerdown', onPointerDown);
    const ancestors = ancestorScroll ? [...(isElement(refs.reference.current) ? reactDom.getOverflowAncestors(refs.reference.current) : []), ...(isElement(refs.floating.current) ? reactDom.getOverflowAncestors(refs.floating.current) : [])] : [];
    ancestors.forEach(ancestor => ancestor.addEventListener('scroll', onScroll, {
      passive: true
    }));
    return () => {
      escapeKey && doc.removeEventListener('keydown', onKeyDown);
      outsidePointerDown && doc.removeEventListener('pointerdown', onPointerDown);
      ancestors.forEach(ancestor => ancestor.removeEventListener('scroll', onScroll));
    };
  }, [escapeKey, outsidePointerDown, events, tree, nodeId, open, onOpenChange, focusReference, ancestorScroll, enabled, refs.floating, refs.reference]);

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onPointerDown() {
        if (referencePointerDown) {
          events.emit('dismiss');
          onOpenChange(false);
        }
      }

    }
  };
};

/**
 * Adds hover event listeners that change the open state, like CSS :focus.
 * @see https://floating-ui.com/docs/useFocus
 */
const useFocus = function (_ref, _temp) {
  let {
    open,
    onOpenChange,
    dataRef,
    refs,
    events
  } = _ref;
  let {
    enabled = true,
    keyboardOnly = true
  } = _temp === void 0 ? {} : _temp;
  const blockFocusRef = React.useRef(false);
  React.useEffect(() => {
    var _doc$defaultView;

    if (!enabled) {
      return;
    }

    const doc = getDocument(refs.floating.current);
    const win = (_doc$defaultView = doc.defaultView) != null ? _doc$defaultView : window;

    function onBlur() {
      blockFocusRef.current = !open;
    }

    function onFocus() {
      setTimeout(() => {
        blockFocusRef.current = false;
      });
    }

    win.addEventListener('focus', onFocus);
    win.addEventListener('blur', onBlur);
    return () => {
      win.addEventListener('focus', onFocus);
      win.removeEventListener('blur', onBlur);
    };
  }, [refs.floating, open, enabled]);
  React.useEffect(() => {
    if (!enabled) {
      return;
    }

    function onDismiss() {
      blockFocusRef.current = true;
    }

    events.on('dismiss', onDismiss);
    return () => {
      events.off('dismiss', onDismiss);
    };
  }, [events, enabled]);

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onPointerDown(_ref2) {
        let {
          pointerType
        } = _ref2;
        blockFocusRef.current = !!(pointerType && keyboardOnly);
      },

      onFocus(event) {
        var _refs$reference$curre, _dataRef$current$open;

        // Note: due to the `window` focus/blur listeners, switching between
        // DevTools touch/normal mode may cause this to fail on the first
        // focus of the window/touch of the element as it gets set to `false`.
        if (blockFocusRef.current) {
          return;
        } // Dismiss with click should ignore the subsequent `focus` trigger, but
        // only if the click originated inside the reference element.


        if (event.type === 'focus' && dataRef.current.openEvent === 'mousedown' && isElement(refs.reference.current) && (_refs$reference$curre = refs.reference.current) != null && _refs$reference$curre.contains((_dataRef$current$open = dataRef.current.openEvent) == null ? void 0 : _dataRef$current$open.target)) {
          return;
        }

        dataRef.current.openEvent = event.nativeEvent;
        onOpenChange(true);
      },

      onBlur(event) {
        var _refs$floating$curren;

        const target = event.relatedTarget; // When focusing the reference element (e.g. regular click), then
        // clicking into the floating element, prevent it from hiding.
        // Note: it must be focusable, e.g. `tabindex="-1"`.

        if ((_refs$floating$curren = refs.floating.current) != null && _refs$floating$curren.contains(target) || isElement(refs.reference.current) && refs.reference.current.contains(target)) {
          return;
        }

        blockFocusRef.current = false;
        onOpenChange(false);
      }

    }
  };
};

const FOCUSABLE_ELEMENT_SELECTOR = 'a[href],area[href],input:not([disabled]),select:not([disabled]),textarea:not([disabled]),button:not([disabled]),iframe,object,embed,*[tabindex],*[contenteditable]';

/**
 * Traps focus in a loop of focusable elements while the floating element is
 * open.
 * @see https://floating-ui.com/docs/useFocusTrap
 */
const useFocusTrap = function (_ref, _temp) {
  let {
    open,
    onOpenChange,
    refs,
    nodeId
  } = _ref;
  let {
    enabled = true,
    initialContentFocus = 0,
    order = ['content'],
    modal = true,
    inert = false,
    scrollLock = false
  } = _temp === void 0 ? {} : _temp;
  const portalId = useFloatingPortalId();
  const tree = useFloatingTree();
  const indexRef = React.useRef(0);
  const getFocusableElements = React.useCallback(() => {
    return order.map(type => {
      if (isHTMLElement(refs.reference.current) && type === 'reference') {
        return refs.reference.current;
      }

      if (refs.floating.current && type === 'floating') {
        return refs.floating.current;
      }

      if (type === 'content') {
        var _refs$floating$curren, _refs$floating$curren2;

        return Array.from((_refs$floating$curren = (_refs$floating$curren2 = refs.floating.current) == null ? void 0 : _refs$floating$curren2.querySelectorAll(FOCUSABLE_ELEMENT_SELECTOR)) != null ? _refs$floating$curren : []);
      }

      return null;
    }).filter(Boolean).flat();
  }, [refs.floating, refs.reference, order]);
  React.useEffect(() => {
    if (!enabled) {
      return;
    }

    const focusableElements = getFocusableElements();

    if (inert) {
      if (open) {
        const first = focusableElements[0];

        if (first === refs.floating.current && !first.contains(first.ownerDocument.activeElement)) {
          first.focus({
            preventScroll: true
          });
        }
      }

      return;
    }

    if (open) {
      if (typeof initialContentFocus === 'number') {
        var _focusableElements$in;

        (_focusableElements$in = focusableElements[initialContentFocus]) == null ? void 0 : _focusableElements$in.focus({
          preventScroll: true
        });
      } else if (initialContentFocus.current) {
        var _focusableElements$fi;

        (_focusableElements$fi = focusableElements.find(element => element === initialContentFocus.current)) == null ? void 0 : _focusableElements$fi.focus({
          preventScroll: true
        });
      }
    } else if (modal && isHTMLElement(refs.reference.current)) {
      refs.reference.current.focus({
        preventScroll: true
      });
    }
  }, [getFocusableElements, open, inert, modal, initialContentFocus, enabled, refs.reference, refs.floating]);
  React.useEffect(() => {
    if (!modal || !enabled) {
      return;
    }

    const doc = getDocument(refs.floating.current);

    if (!open) {
      doc.removeEventListener('keydown', onKeyDown);
      indexRef.current = 0;
      return;
    }

    function onKeyDown(event) {
      var _tree$nodesRef$curren;

      if (tree != null && (_tree$nodesRef$curren = tree.nodesRef.current) != null && _tree$nodesRef$curren.filter(_ref2 => {
        let {
          parentId
        } = _ref2;
        return parentId === nodeId;
      }).some(_ref3 => {
        let {
          context
        } = _ref3;
        return context == null ? void 0 : context.open;
      })) {
        return;
      }

      if (event.key === 'Tab') {
        var _focusableElements$in2;

        event.preventDefault();
        event.stopPropagation();
        const focusableElements = getFocusableElements();

        if (inert) {
          return;
        }

        if (event.shiftKey) {
          indexRef.current = indexRef.current === 0 ? focusableElements.length - 1 : indexRef.current - 1;
        } else {
          indexRef.current = indexRef.current === focusableElements.length - 1 ? 0 : indexRef.current + 1;
        }

        (_focusableElements$in2 = focusableElements[indexRef.current]) == null ? void 0 : _focusableElements$in2.focus({
          preventScroll: true
        });
      }
    }

    doc.addEventListener('keydown', onKeyDown);
    return () => {
      doc.removeEventListener('keydown', onKeyDown);
    };
  }, [getFocusableElements, tree == null ? void 0 : tree.nodesRef, nodeId, open, modal, inert, enabled, refs.floating]);

  function onBlur(event) {
    var _refs$floating$curren3;

    const target = event.relatedTarget;

    if (!((_refs$floating$curren3 = refs.floating.current) != null && _refs$floating$curren3.contains(target)) && isElement(refs.reference.current) && !refs.reference.current.contains(target)) {
      onOpenChange(false);
    }
  }

  React.useEffect(() => {
    if (!open || !modal || !enabled) {
      return;
    }

    const doc = getDocument(refs.floating.current);
    const portal = doc.querySelector("#" + portalId);
    const nodes = doc.querySelectorAll("body > *:not(#" + portalId + ")");
    nodes.forEach(node => {
      node.setAttribute('aria-hidden', 'true');
    });
    return () => {
      if ((portal == null ? void 0 : portal.firstElementChild) === refs.floating.current) {
        nodes.forEach(node => {
          node.removeAttribute('aria-hidden');
        });
      }
    };
  }, [open, modal, scrollLock, portalId, enabled, refs.floating]);
  index(() => {
    if (!open || !enabled) {
      return;
    }

    const doc = getDocument(refs.floating.current);

    if (scrollLock) {
      var _doc$defaultView;

      const scrollbarWidth = ((_doc$defaultView = doc.defaultView) != null ? _doc$defaultView : window).innerWidth - doc.documentElement.offsetWidth;
      doc.documentElement.style.overflow = 'hidden';
      doc.body.style.paddingRight = scrollbarWidth + "px";
    }

    return () => {
      if (scrollLock) {
        doc.documentElement.style.overflow = '';
        doc.body.style.paddingRight = '';
      }
    };
  }, [open, scrollLock, enabled]);

  if (!enabled) {
    return {};
  }

  if (modal) {
    return {
      floating: {
        'aria-modal': 'true'
      }
    };
  }

  return {
    reference: {
      onBlur
    },
    floating: {
      onBlur
    }
  };
};

const ARROW_UP = 'ArrowUp';
const ARROW_DOWN = 'ArrowDown';
const ARROW_LEFT = 'ArrowLeft';
const ARROW_RIGHT = 'ArrowRight';

function stop(event) {
  event.preventDefault();
  event.stopPropagation();
}

function findNonDisabledIndex(listRef, _temp) {
  let {
    startingIndex = -1,
    decrement = false
  } = _temp === void 0 ? {} : _temp;
  let index = startingIndex;

  do {
    var _listRef$current$inde, _listRef$current$inde2;

    index = index + (decrement ? -1 : 1);
  } while ((_listRef$current$inde = listRef.current[index]) != null && _listRef$current$inde.hasAttribute('disabled') || ((_listRef$current$inde2 = listRef.current[index]) == null ? void 0 : _listRef$current$inde2.getAttribute('aria-disabled')) === 'true');

  return index === -1 ? 0 : index;
}

function doSwitch(orientation, vertical, horizontal) {
  switch (orientation) {
    case 'vertical':
      return vertical;

    case 'horizontal':
      return horizontal;

    default:
      return vertical || horizontal;
  }
}

function isMainorientationKey(key, orientation) {
  const vertical = key === ARROW_UP || key === ARROW_DOWN;
  const horizontal = key === ARROW_LEFT || key === ARROW_RIGHT;
  return doSwitch(orientation, vertical, horizontal);
}

function isMainorientationToStartKey(key, orientation, rtl) {
  const vertical = key === ARROW_UP;
  const horizontal = rtl ? key === ARROW_RIGHT : key === ARROW_LEFT;
  return doSwitch(orientation, vertical, horizontal);
}

function isMainorientationToEndKey(key, orientation, rtl) {
  const vertical = key === ARROW_DOWN;
  const horizontal = rtl ? key === ARROW_LEFT : key === ARROW_RIGHT;
  return doSwitch(orientation, vertical, horizontal);
}

function isCrossorientationOpenKey(key, orientation, rtl) {
  const vertical = rtl ? key === ARROW_LEFT : key === ARROW_RIGHT;
  const horizontal = key === ARROW_DOWN;
  return doSwitch(orientation, vertical, horizontal);
}

function isCrossorientationCloseKey(key, orientation, rtl) {
  const vertical = rtl ? key === ARROW_RIGHT : key === ARROW_LEFT;
  const horizontal = key === ARROW_UP;
  return doSwitch(orientation, vertical, horizontal);
}

function getMinIndex(listRef) {
  return findNonDisabledIndex(listRef);
}

function getMaxIndex(listRef) {
  return findNonDisabledIndex(listRef, {
    decrement: true,
    startingIndex: listRef.current.length
  });
}

/**
 * Adds focus-managed indexed navigation via arrow keys to a list of items
 * within the floating element.
 * @see https://floating-ui.com/docs/useListNavigation
 */
const useListNavigation = function (_ref, _temp2) {
  let {
    open,
    onOpenChange,
    refs
  } = _ref;
  let {
    listRef,
    activeIndex,
    onNavigate,
    enabled = true,
    selectedIndex = null,
    loop = false,
    nested = false,
    rtl = false,
    focusItemOnOpen = 'auto',
    orientation = 'vertical'
  } = _temp2 === void 0 ? {
    listRef: {
      current: []
    },
    activeIndex: null,
    onNavigate: () => {}
  } : _temp2;
  const focusOnOpenRef = React.useRef(focusItemOnOpen);
  const indexRef = React.useRef(selectedIndex != null ? selectedIndex : -1);
  const keyRef = React.useRef('');
  const focusItem = React.useCallback((listRef, indexRef) => {
    var _listRef$current$inde3;

    (_listRef$current$inde3 = listRef.current[indexRef.current]) == null ? void 0 : _listRef$current$inde3.focus({
      preventScroll: true
    });
  }, []);
  index(() => {
    if (!enabled) {
      return;
    }

    if (selectedIndex != null) {
      indexRef.current = selectedIndex;
    }

    if (open && focusOnOpenRef.current) {
      onNavigate(indexRef.current);
      focusItem(listRef, indexRef);
    }
  }, [open, selectedIndex, listRef, onNavigate, focusItem, enabled]);
  index(() => {
    if (!enabled) {
      return;
    }

    if (open && activeIndex != null) {
      indexRef.current = activeIndex;
      onNavigate(indexRef.current);
      focusItem(listRef, indexRef);
    }
  }, [open, activeIndex, listRef, onNavigate, focusItem, enabled]);
  index(() => {
    if (selectedIndex != null || !enabled) {
      return;
    }

    if (open) {
      if (isMainorientationKey(keyRef.current, orientation) || focusOnOpenRef.current && (keyRef.current === ' ' || keyRef.current === 'Enter')) {
        indexRef.current = isMainorientationToStartKey(keyRef.current, orientation, rtl) ? getMaxIndex(listRef) : getMinIndex(listRef);
        onNavigate(indexRef.current);
        focusItem(listRef, indexRef);
      }
    }

    keyRef.current = '';
  }, [open, listRef, selectedIndex, onNavigate, focusItem, enabled]);
  index(() => {
    if (!enabled) {
      return;
    }

    if (!open && selectedIndex != null) {
      var _refs$reference$curre;

      (_refs$reference$curre = refs.reference.current) == null ? void 0 : _refs$reference$curre.focus({
        preventScroll: true
      });
    }
  }, [refs.reference, selectedIndex, open, enabled]);
  index(() => {
    if (!enabled) {
      return;
    }

    if (!open) {
      var _ref2;

      if (focusItemOnOpen === 'auto') {
        focusOnOpenRef.current = true;
      }

      indexRef.current = (_ref2 = selectedIndex != null ? selectedIndex : activeIndex) != null ? _ref2 : -1;
      onNavigate(null);
    }
  }, [open, selectedIndex, activeIndex, enabled]);

  function pointerCheck(event) {
    if (focusItemOnOpen === 'auto') {
      // undefined or '' depending on the browser
      focusOnOpenRef.current = !event.pointerType;
    }
  }

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onPointerEnter: pointerCheck,
      onPointerDown: pointerCheck,

      onKeyDown(event) {
        if (focusItemOnOpen === 'auto') {
          focusOnOpenRef.current = true;
        }

        keyRef.current = event.key;

        if (isElement(refs.reference.current) && refs.reference.current.tagName !== 'BUTTON') {
          if (event.key === 'Enter' || event.key === ' ') {
            onOpenChange(true);
          }
        }

        if (nested) {
          if (isCrossorientationOpenKey(event.key, orientation, rtl)) {
            indexRef.current = getMinIndex(listRef);
            stop(event);
            onOpenChange(true);
            onNavigate(indexRef.current);
          }

          return;
        }

        if (isMainorientationKey(event.key, orientation)) {
          if (selectedIndex == null) {
            indexRef.current = isMainorientationToEndKey(event.key, orientation, rtl) ? getMinIndex(listRef) : getMaxIndex(listRef);
          } else {
            indexRef.current = selectedIndex;
          }

          stop(event);
          onOpenChange(true);
          onNavigate(indexRef.current);
        }
      }

    },
    floating: {
      'aria-orientation': orientation === 'both' ? undefined : orientation,

      onKeyDown(event) {
        if (nested && isCrossorientationCloseKey(event.key, orientation, rtl)) {
          stop(event);
          onOpenChange(false);

          if (isHTMLElement(refs.reference.current)) {
            refs.reference.current.focus();
          }

          return;
        }

        const currentIndex = indexRef.current;
        const minIndex = getMinIndex(listRef);
        const maxIndex = getMaxIndex(listRef);

        if (event.key === 'Home') {
          indexRef.current = minIndex;
          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }

        if (event.key === 'End') {
          indexRef.current = maxIndex;
          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }

        if (isMainorientationKey(event.key, orientation)) {
          stop(event);

          if (event.currentTarget.ownerDocument.activeElement === event.currentTarget) {
            indexRef.current = selectedIndex != null ? selectedIndex : isMainorientationToEndKey(event.key, orientation, rtl) ? minIndex : maxIndex;
            onNavigate(indexRef.current);
            focusItem(listRef, indexRef);
            return;
          }

          if (isMainorientationToEndKey(event.key, orientation, rtl)) {
            if (loop) {
              indexRef.current = currentIndex === maxIndex ? minIndex : findNonDisabledIndex(listRef, {
                startingIndex: currentIndex
              });
            } else {
              indexRef.current = Math.min(maxIndex, findNonDisabledIndex(listRef, {
                startingIndex: currentIndex
              }));
            }
          } else {
            if (loop) {
              indexRef.current = currentIndex === minIndex ? maxIndex : findNonDisabledIndex(listRef, {
                startingIndex: currentIndex,
                decrement: true
              });
            } else {
              indexRef.current = Math.max(minIndex, findNonDisabledIndex(listRef, {
                startingIndex: currentIndex,
                decrement: true
              }));
            }
          }

          onNavigate(indexRef.current);
          focusItem(listRef, indexRef);
        }
      }

    }
  };
};

/**
 * Provides a matching callback that can be used to focus an item as the user
 * types, often used in tandem with `useListNavigation()`.
 * @see https://floating-ui.com/docs/useTypeahead
 */
const useTypeahead = function (_ref, _temp) {
  var _ref2;

  let {
    open
  } = _ref;
  let {
    listRef,
    activeIndex,
    onMatch = () => {},
    enabled = true,
    onFind = null,
    debounceMs = 500,
    ignoreKeys = [],
    selectedIndex = null
  } = _temp === void 0 ? {
    listRef: {
      current: []
    },
    activeIndex: null
  } : _temp;
  const timeoutIdRef = React.useRef();
  const stringRef = React.useRef('');
  const prevIndexRef = React.useRef((_ref2 = selectedIndex != null ? selectedIndex : activeIndex) != null ? _ref2 : -1);
  const matchIndexRef = React.useRef(null);
  index(() => {
    if (open) {
      var _ref3;

      clearTimeout(timeoutIdRef.current);
      prevIndexRef.current = (_ref3 = selectedIndex != null ? selectedIndex : activeIndex) != null ? _ref3 : -1;
      matchIndexRef.current = null;
      stringRef.current = '';
    }
  }, [open, selectedIndex, activeIndex]);

  function onKeyDown(event) {
    if (!event.currentTarget.contains(getDocument(event.currentTarget).activeElement)) {
      return;
    }

    if (event.key === ' ' && stringRef.current.length > 0) {
      event.preventDefault();
      event.stopPropagation();
    }

    const listContent = listRef.current;

    if (listContent == null || ['Home', 'End', 'Escape', 'Enter', 'Tab', 'ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight', ...ignoreKeys].includes(event.key)) {
      return;
    } // Bail out if the list contains a word like "llama" or "aaron". TODO:
    // allow it in this case, too.


    const allowRapidSuccessionOfFirstLetter = listContent.every(text => {
      var _text$, _text$2;

      return text ? ((_text$ = text[0]) == null ? void 0 : _text$.toLowerCase()) !== ((_text$2 = text[1]) == null ? void 0 : _text$2.toLowerCase()) : true;
    }); // Allows the user to cycle through items that start with the same letter
    // in rapid succession

    if (allowRapidSuccessionOfFirstLetter && stringRef.current === event.key) {
      stringRef.current = '';
      prevIndexRef.current = matchIndexRef.current;
    }

    stringRef.current += event.key;
    clearTimeout(timeoutIdRef.current);
    timeoutIdRef.current = setTimeout(() => {
      stringRef.current = '';
      prevIndexRef.current = matchIndexRef.current;
    }, debounceMs);
    const prevIndex = prevIndexRef.current;
    const str = [...listContent.slice((prevIndex != null ? prevIndex : 0) + 1), ...listContent.slice(0, prevIndex != null ? prevIndex : 0)].find(text => onFind ? onFind(text, stringRef.current) : (text == null ? void 0 : text.toLowerCase().indexOf(stringRef.current)) === 0);
    const index = str ? listContent.indexOf(str) : -1;

    if (index !== -1) {
      onMatch(index);
      matchIndexRef.current = index;
    }
  }

  if (!enabled) {
    return {};
  }

  return {
    reference: {
      onKeyDown
    },
    floating: {
      onKeyDown
    }
  };
};

function useFloating(_temp) {
  let {
    open = false,
    onOpenChange = () => {},
    placement,
    middleware,
    strategy,
    nodeId
  } = _temp === void 0 ? {} : _temp;
  const tree = useFloatingTree();
  const dataRef = React.useRef({});
  const events = React.useState(() => createPubSub())[0];
  const floating = reactDom.useFloating({
    placement,
    middleware,
    strategy
  });
  const context = React.useMemo(() => ({ ...floating,
    dataRef,
    nodeId,
    events,
    open,
    onOpenChange
  }), [floating, dataRef, nodeId, events, open, onOpenChange]);
  index(() => {
    const node = tree == null ? void 0 : tree.nodesRef.current.find(node => node.id === nodeId);

    if (node) {
      node.context = context;
    }
  });
  return React.useMemo(() => ({
    context,
    ...floating
  }), [floating, context]);
}

exports.FloatingDelayGroup = FloatingDelayGroup;
exports.FloatingNode = FloatingNode;
exports.FloatingOverlay = FloatingOverlay;
exports.FloatingPortal = FloatingPortal;
exports.FloatingTree = FloatingTree;
exports.safePolygon = safePolygon;
exports.useClick = useClick;
exports.useDelayGroup = useDelayGroup;
exports.useDelayGroupContext = useDelayGroupContext;
exports.useDismiss = useDismiss;
exports.useFloating = useFloating;
exports.useFloatingId = useFloatingId;
exports.useFloatingNodeId = useFloatingNodeId;
exports.useFloatingParentNodeId = useFloatingParentNodeId;
exports.useFloatingTree = useFloatingTree;
exports.useFocus = useFocus;
exports.useFocusTrap = useFocusTrap;
exports.useHover = useHover;
exports.useInteractions = useInteractions;
exports.useListNavigation = useListNavigation;
exports.useRole = useRole;
exports.useTypeahead = useTypeahead;
Object.keys(reactDom).forEach(function (k) {
  if (k !== 'default' && !exports.hasOwnProperty(k)) Object.defineProperty(exports, k, {
    enumerable: true,
    get: function () { return reactDom[k]; }
  });
});
